/**
  * BMPViewer - Simple test frame to Load, Display & Save BMP images.
  *
  * @author Richard J.Osbaldeston
  * @version 1.0 26/06/98
  * @copyright Richard J.Osbaldeston (http://www.osbald.co.uk)
  */
  
import java.awt.*;
import java.awt.event.*;
import osbaldeston.image.BMP;

public class BMPViewer extends Frame implements ActionListener {

    private ImagePanel imagePanel = new ImagePanel();
    private MediaTracker tracker = new MediaTracker(this);
    private FileDialog fileDialog = new FileDialog(this, "Select File");

    public BMPViewer() {
        setTitle("BMP Viewer - by R.J.Osbaldeston 1998");
        setForeground(Color.black);
        setBackground(Color.lightGray);
        
        Menu menu = new Menu("File");
        MenuBar menubar= new MenuBar();
        menubar.add(menu);
        menu.add(new MenuItem("Load"));
        menu.add(new MenuItem("Save"));
        menu.add(new MenuItem("Exit"));
        setMenuBar(menubar);
        menu.addActionListener(this);
        add(BorderLayout.CENTER, imagePanel);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
        }});
        pack();
        show();
    }
    
    public void load() {
      // show filedialog
      fileDialog.setMode(fileDialog.LOAD);
      fileDialog.setVisible(true);
      
      // load image, use bmp classes if it has a bmp extension
      if (fileDialog.getFile() != null) {
        Image image = null;
        String filename = fileDialog.getDirectory()+fileDialog.getFile();
        if (filename.endsWith(".bmp") || filename.endsWith(".BMP")) {
            BMP bmp = new BMP(new java.io.File(filename));
            image = bmp.getImage();
        } else {
            image = Toolkit.getDefaultToolkit().getImage(filename);
            try {
                tracker.addImage(image, 0);
                tracker.waitForID(0);
				tracker.removeImage(image);
            } catch(InterruptedException e) {
                System.err.println("*ERROR* MediaTracker rudely interrupted.");
                image=null;
            }
        }
        if (image != null && image.getHeight(null)>=0) {
            imagePanel.setImage(image);
        } else  {
	        System.err.println("*ERROR* Couldn't load image:"+filename);
        }
        pack();
      }
    }
    
    public void save() {
      // show filedialog
      fileDialog.setMode(fileDialog.SAVE);
      fileDialog.setVisible(true);
      // save the image in bmp format. NB: WILL overwrite exisiting files.
      if (fileDialog.getFile() != null) {
        String filename = fileDialog.getDirectory()+fileDialog.getFile();
        Image image = imagePanel.getImage();
        if (image != null) {
            BMP bmp = new BMP(image);
            bmp.write(new java.io.File(filename));
        }
      }
    }   

    // handle menu events
    public void actionPerformed(ActionEvent e) {
        String label = e.getActionCommand();
        if (label.equals("Load"))  {
            load();
        }        
        else if (label.equals("Save")) {
            save();
        }
        else if (label.equals("Exit")) {
           dispose();
        }
    }

    // Simple utility class (inner class) used to display an Image.
    private class ImagePanel extends Panel {
        private Image image;
        
        public synchronized void setImage(Image image) {
            if (this.image != null) {
                this.image.flush();
                this.image=null;
            }
            this.image=image;
            setSize(image.getWidth(this),image.getHeight(this));
            invalidate();
            repaint();            
        }

        public synchronized Image getImage() {
            return image;
        }
        
        public Dimension getPreferredSize() {
            if (image != null) {
                return new Dimension(image.getWidth(this),image.getHeight(this));
            } else {
                return new Dimension(320,160);
            }
        }

        public void update(Graphics g) {
            paint(g);
        }

        public void paint(Graphics g) {
            if (image != null) {
                g.drawImage(image, 0, 0, this);
            }
            super.paint(g);
        }
    }

    public static void main (String args[]) {
        new BMPViewer();
    }
}